package scancode;

import java.io.File;

/**
 * Copyright (C) 2010  Molly Megraw
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: molly.megraw@duke.edu
 *
 */
public class Background {
    public Background() {
    }

    public static void main(String[] args) throws java.io.IOException {
        Background background = new Background();

        // Define input files
        String inFile = args[0];

        // Define output files
        String outFile = args[1];
        String outFile2 = args[2];

        // Read input file in Fasta format
        FastaReturn fa = Load.loadFastaFile(new File(inFile));
        char[][] S = new char[fa.lines.length][];
        for (int i = 0; i < fa.lines.length; i++) {
            S[i] = fa.lines[i].toCharArray();
        }

        double bg_m0[] = getBackground_M0(S);
        Print.printArrayToFile(bg_m0, outFile);

        double bg_m1[][] = getBackground_M1(S);
        Print.printMatrixToFile(bg_m1, outFile2);
    }

    public static double[] getBackground_M0(char[][] S) {

        double[] Theta_o = new double[Setup.NCHARS];
        int chrInd;
        double sum;

        // Initialize Theta_o
        for (int k = 0; k < Theta_o.length; k++) {
            Theta_o[k] = 0.0;
        }

        // Tally background bases
        for (int i = 0; i < S.length; i++) {
            for (int j = 0; j < S[i].length; j++) {
                chrInd = Setup.charInd(S[i][j]);
                if (!(chrInd < 0)) { // recognized character
                    Theta_o[chrInd] += 1.0;
                }
            }
        }

        // Divide by total
        sum = 0.0;
        for (int k = 0; k < Theta_o.length; k++) {
            sum += Theta_o[k];
        }
        for (int k = 0; k < Theta_o.length; k++) {
            Theta_o[k] /= sum;
        }

        return Theta_o;

    }

    public static double[][] getBackground_M1(char[][] S) {

        double[][] Theta_o_M1 = new double[Setup.NCHARS][Setup.NCHARS];

        int chrInd1, chrInd2;
        double[] sum = new double[Setup.NCHARS];

        // Initialize Theta_o_M1
        for (int k = 0; k < Theta_o_M1.length; k++) {
            for (int k2 = 0; k2 < Theta_o_M1[k].length; k2++) {
                Theta_o_M1[k][k2] = 0.0;
            }
        }

        // Tally background di-nucleotide pairs
        for (int i = 0; i < S.length; i++) {
            for (int j = 0; j < S[i].length - 1; j++) {
                chrInd1 = Setup.charInd(S[i][j]);
                chrInd2 = Setup.charInd(S[i][j+1]);
                if (!(chrInd1 < 0) && !(chrInd2 < 0))  { // recognized characters
                    Theta_o_M1[chrInd1][chrInd2] += 1.0;
                }
            }
        }

        // Divide by row totals
        for (int k = 0; k < Theta_o_M1.length; k++) {
            sum[k] = 0.0;
            for (int k2 = 0; k2 < Theta_o_M1[k].length; k2++) {
                sum[k] += Theta_o_M1[k][k2];
            }
        }
        for (int k = 0; k < Theta_o_M1.length; k++) {
            for (int k2 = 0; k2 < Theta_o_M1[k].length; k2++) {
                Theta_o_M1[k][k2] /= sum[k];
            }
        }

        return Theta_o_M1;

    }

}
