package scancode;

import java.io.File;
import java.io.PrintWriter;
import java.io.FileWriter;

/**
 * Copyright (C) 2010  Molly Megraw
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: molly.megraw@duke.edu
 *
 */
public class Thresholds {
    public Thresholds() {
    }

    public static void main(String[] args) throws java.io.IOException {
        Thresholds scansites = new Thresholds();

        PWMReturn pwms;
        double[] bg;

        double PseudoCountsVal = Double.parseDouble(args[0]);
        String pwms_Fname = args[1];
        String bg_Fname = args[2];
        String seqs_Fname = args[3];
        String out_Fname = args[4];

        // Read PWM file
        pwms = Load.loadPWMFile(pwms_Fname, PseudoCountsVal);

        // Read background file
        bg = Load.loadVector(new File(bg_Fname));

        // Read sites file
        SeqArrayReturn Seqs = Load.loadSeqArrayFile(seqs_Fname);

        // Open file for printing count observations
        PrintWriter outFile = new PrintWriter(new FileWriter(out_Fname));

        // Scan each PWM over each sequence set
        double[][] ThetaT;
        for (int nmat = 0; nmat < pwms.pwms.length; nmat++) {
            ThetaT = pwms.pwms[nmat];

            if (!(ThetaT.length == Seqs.seqs[nmat][0].length)) {
                System.out.println("ERROR: width for PWM labeled " + pwms.labels[nmat] + " and sites labeled " + Seqs.labels[nmat] + " do not agree.");
                System.exit(1);
            }

            double minScore = getThreshold(Seqs.seqs[nmat], ThetaT, bg);
            outFile.println(Seqs.labels[nmat] + "\t" + Print.df3.format(minScore));
        }

        outFile.close();
    }

    public static double getThreshold (char[][] S, double[][] ThetaT, double[] B) {

        double LARGE = 1e6;
        int w = ThetaT.length;
        int chrInd;
        double theta_freq, bg_freq, score, minScore;
        double numLogSum, denLogSum;
        boolean acceptMotif;

        // Compute scores for each site
        minScore = LARGE;
        for (int i = 0; i < S.length; i++) {
            // Take product of freq/bg probabilities over site width
            numLogSum = 0.0;
            denLogSum = 0.0;
            acceptMotif = true;
            for (int mp = 0; mp < w; mp++) {
                chrInd = Setup.charInd(S[i][mp]);
                if (chrInd == -1) {
                    acceptMotif = false;
                    break;
                }
                theta_freq = ThetaT[mp][chrInd];
                bg_freq = B[chrInd];
                numLogSum += Math.log(theta_freq);
                denLogSum += Math.log(bg_freq);
            }

            // Only consider site if all characters are recognized
            if (acceptMotif == true) {
                score = numLogSum - denLogSum;
                if (score < minScore) {
                    minScore = score;
                }
            }
        }

        return minScore;

    }

}
